@@ -4,6 +4,13 @@ import os.path
 import itertools
 
 
+try:
+    from collections import OrderedDict
+except ImportError:
+    # Python < 2.7
+    from ordereddict import OrderedDict
+
+
 class SpaceList(list):
     def __add__(self, other):
         return SpaceList(super(SpaceList, self).__add__(other))
@@ -15,9 +22,7 @@ class SpaceList(list):
         return SpaceList(getattr(x, 'path', x) for x in self)
 
 
-class FileMap(dict):
-    __iter__ = dict.iteritems
-
+class FileMap(OrderedDict):
     def sources(self):
         return SpaceList(self.iterkeys())
 
@@ -25,7 +30,7 @@ class FileMap(dict):
         return SpaceList(self.itervalues())
 
     def iterpaths(self):
-        for source, target in self:
+        for source, target in self.iteritems():
             yield (source.path, target.path)
 
     def target_paths(self):
